/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.test.ws.jaxws.samples.serviceref;

import java.io.File;
import java.io.InputStream;
import java.net.URL;

import javax.naming.InitialContext;
import javax.xml.namespace.QName;
import javax.xml.ws.Service;

import junit.framework.Test;

import org.jboss.test.ws.jaxws.samples.serviceref.EJBRemote;
import org.jboss.wsf.test.JBossWSTest;
import org.jboss.wsf.test.JBossWSTestHelper;
import org.jboss.wsf.test.JBossWSTestSetup;

/**
 * Test the JAXRPC <service-ref>
 *
 * @author Thomas.Diesler@jboss.com
 * @since 23-Oct-2005
 */
public class ServiceRefEJBTestCase extends JBossWSTest
{
   public final String TARGET_ENDPOINT_ADDRESS = "http://" + getServerHost() + ":8080/jaxws-samples-serviceref";
   
   public static Test suite()
   {
      String archives = "jaxws-samples-serviceref.war";
      if (new JBossWSTestHelper().isTargetJBoss50())
         archives += ",jaxws-samples-serviceref-ejb-client.jar";
      
      return new JBossWSTestSetup(ServiceRefEJBTestCase.class, archives);
   }

   public void testWSDLAccess() throws Exception
   {
      URL wsdlURL = new URL(TARGET_ENDPOINT_ADDRESS + "?wsdl");
      InputStream inputStream = wsdlURL.openStream();
      assertNotNull(inputStream);
      inputStream.close();
   }

   
   public void testDynamicProxy() throws Exception
   {
      // JAXWS <service-ref> is only supported in EE5 
      if (isTargetJBoss50())
      {
         URL wsdlURL = getResourceURL("jaxws/samples/serviceref/META-INF/wsdl/Endpoint.wsdl");
         QName qname = new QName("http://serviceref.samples.jaxws.ws.test.jboss.org/", "EndpointService");
         Service service = Service.create(wsdlURL, qname);
         Endpoint port = (Endpoint)service.getPort(Endpoint.class);

         String helloWorld = "DynamicProxy";
         Object retObj = port.echo(helloWorld);
         assertEquals(helloWorld, retObj);
      }
   }


   public void testEJBClient() throws Exception
   {      
      // JAXWS <service-ref> is only supported in EE5 
      if (isTargetJBoss50())
      {
         InitialContext iniCtx = getInitialContext();
         EJBRemote ejbRemote = (EJBRemote)iniCtx.lookup("/EJBClient/remote");

         String helloWorld = "Hello World!";
         Object retObj = ejbRemote.echo(helloWorld);
         assertEquals(helloWorld, retObj);
      }
   }
}
